import type { Container } from "./Container";
import type { IParticleImage } from "./Interfaces/IParticleImage";
import { Updater } from "./Particle/Updater";
import type { IParticles } from "../Options/Interfaces/Particles/IParticles";
import { ParticlesOptions } from "../Options/Classes/Particles/ParticlesOptions";
import { ShapeType } from "../Enums";
import type { RecursivePartial } from "../Types";
import { Mover } from "./Particle/Mover";
import type { ILink } from "./Interfaces/ILink";
import type { Stroke } from "../Options/Classes/Particles/Stroke";
import { Vector } from "./Particle/Vector";
import type { IBubbleParticleData, ICoordinates, IDelta, IDistance, IHsl, IParticle, IParticleHslAnimation, IParticleLife, IParticleLoops, IParticleNumericValueAnimation, IParticleTiltValueAnimation, IParticleValueAnimation, IRgb, IShapeValues } from "./Interfaces";
import { Vector3d } from "./Particle/Vector3d";
export declare class Particle implements IParticle {
    readonly id: number;
    private readonly container;
    readonly group?: string | undefined;
    destroyed: boolean;
    lastPathTime: number;
    misplaced: boolean;
    rollAngle: number;
    rollSpeed: number;
    spawning: boolean;
    splitCount: number;
    unbreakable: boolean;
    wobbleAngle: number;
    wobbleDistance: number;
    wobbleSpeed: number;
    readonly pathDelay: number;
    readonly updater: Updater;
    readonly mover: Mover;
    readonly sides: number;
    readonly strokeWidth: number;
    readonly options: ParticlesOptions;
    readonly life: IParticleLife;
    readonly loops: IParticleLoops;
    readonly maxDistance: Partial<IDistance>;
    attractDistance?: number;
    backColor?: IHsl;
    links: ILink[];
    randomIndexData?: number;
    linksDistance?: number;
    linksWidth?: number;
    moveDrift?: number;
    moveSpeed?: number;
    maxSpeed?: number;
    sizeAnimationSpeed?: number;
    readonly close: boolean;
    readonly direction: number;
    readonly fill: boolean;
    readonly stroke: Stroke;
    readonly position: Vector3d;
    readonly offset: Vector;
    readonly shadowColor: IRgb | undefined;
    readonly color?: IParticleHslAnimation;
    readonly opacity: IParticleNumericValueAnimation;
    readonly rotate: IParticleValueAnimation<number>;
    readonly size: IParticleNumericValueAnimation;
    readonly tilt: IParticleTiltValueAnimation;
    readonly strokeColor?: IParticleHslAnimation;
    readonly velocity: Vector;
    readonly shape: ShapeType | string;
    readonly image?: IParticleImage;
    readonly initialPosition: Vector;
    readonly initialVelocity: Vector;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    readonly zIndexFactor: number;
    constructor(id: number, container: Container, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticles>, group?: string | undefined);
    move(delta: IDelta): void;
    update(delta: IDelta): void;
    draw(delta: IDelta): void;
    getPosition(): ICoordinates;
    getRadius(): number;
    getMass(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(override?: boolean): void;
    reset(): void;
    private split;
    private setColorAnimation;
    private calcPosition;
    private checkOverlap;
    private calculateVelocity;
    private loadImageShape;
    private loadLife;
}
